#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str
import time

from pynamodb.exceptions import GetError

from db_paribhasha import credentials
from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.dynamo.export_adapter import ExportAdapter
from db_paribhasha.error_handler import (DynamoDBError,
                                         OperationNotPerformedError)


def create_table():
    """
    Creates table if table doesn't exist
    """

    if ExportAdapter.exists():
        raise OperationNotPerformedError("Table already exists")
    ExportAdapter.create_table(
        read_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP,
        write_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP)


def add_adapter(org_id, adapter_definition, display_info, default=False):
    """
    Adds an export adapter for an organizaion ID.

    Parameters
    ---------
    org_id: Integer
        ID of the Organization to add the export adapter
    adapter_definition: JSON
        JSON containing the adapter definition
    default: Boolean
        Boolean specifying whether this is the default adapter of the org
        export.

    Returns
    -------
    id: Integer
        ID of the created export adapter.
    """
    added_timestamp = int(round(time.time() * 1000))
    last_modified_timestamp = added_timestamp
    calculated_id = '{}-{}'.format(str(org_id), str(added_timestamp))
    added_adapter =\
        ExportAdapter(org_id=int(org_id), added_timestamp=added_timestamp,
                      calculated_id=calculated_id,
                      last_modified_timestamp=last_modified_timestamp,
                      export_definition=adapter_definition,
                      display_info=display_info,
                      default_adapter=default)
    db_helpers.save_dynamo_object(added_adapter)
    return calculated_id


def get_adapter(adapter_id):
    """
    Fetches the adapter for a given unique adapter id

    Parameters
    ----------
    adapter_id: String
        Unique Adapter ID

    Returns
    -------
    adapter: Pynamo Model Object
        An item from the ExportAdapter table
    """
    org_id, added_timestamp = adapter_id.split('-')
    try:
        adapter = ExportAdapter.get(int(org_id), int(added_timestamp))
    except GetError as err:
        raise DynamoDBError(err)
    return adapter


def update_adapter(adapter_id, new_definition, default=False):
    """
    Updates a given adapter using adapter_id

    Parameters
    ----------
    adapter_id: String
        Unique Adapter ID
    new_definition: Dict
        Dict loaded from JSON
    default: Boolean
        Specifies whether the adapter is default or not
    """
    adapter = get_adapter(adapter_id)
    adapter.export_definition = new_definition
    adapter.default_adapter = default
    adapter.last_modified_timestamp = int(round(time.time() * 1000))
    db_helpers.save_dynamo_object(adapter)


def get_adapters_for_org(org_id):
    """
    Returns all the adapters for the given  organization ID.

    Parameters
    ----------
    org_id: Integer
        Organization ID to query for adapters

    Returns
    -------
    adapters
    """
    try:
        adapters = ExportAdapter.query(int(org_id))
    except GetError as err:
        raise DynamoDBError(err)
    return adapters


def get_default_adapter(org_id):
    """
    Returns default adapters for the org using GSI

    Parameters
    ----------
    org_id: Integer
        Organization ID to query for default adapter

    Returns
    -------
    """
    try:
        adapter = None
        for adapter in ExportAdapter.query(
                int(org_id), ExportAdapter.default_adapter == True):
            return adapter
    except GetError as err:
        raise DynamoDBError(err)
    raise DynamoDBError('No Default adapter defined')
